use std::{collections::HashMap};

use slint::{ModelRc, VecModel};

use super::{OpCode, Script};
use crate::{Node, Port};

pub struct NodeTemplate {
    pub name: String,
    pub script: Script, // TODO: figure out what to do with this?
    pub input_connections: usize,
    pub output_connections: usize,
}

impl NodeTemplate {
    pub fn to_slint_node(self) -> Node {
        let mut in_port_vec = vec![];
        let mut out_port_vec = vec![];

        for i in 0..self.input_connections {
            in_port_vec.push(Port {
                name: format!("i{i}").into(),
                ..Default::default()
            });
        }

        for j in 0..self.output_connections {
            out_port_vec.push(Port {
                name: format!("o{j}").into(),
                ..Default::default()
            });
        }

        Node {
            text: self.name.into(),
            x: 0.0, //?
            y: 0.0, //?
            in_ports: ModelRc::new(VecModel::from(in_port_vec)),
            out_ports: ModelRc::new(VecModel::from(out_port_vec)),
            ..Default::default()
        }
    }
}

pub fn make_mixer(input_count: usize) -> NodeTemplate {
    NodeTemplate {
        name: "Mixer".to_owned(),
        script: Script::from_raw_bytecode(vec![
            OpCode::Add,
            OpCode::JumpIfStackSizeEq(3, 1),
            OpCode::Jump(0),
            OpCode::Brk,
        ]),
        input_connections: input_count,
        output_connections: 1,
    }
}

pub fn make_splitter(output_count: usize) -> NodeTemplate {
    NodeTemplate {
        name: "Splitter".to_owned(),
        script: Script::from_raw_bytecode(vec![
            OpCode::Dup,
            OpCode::JumpIfStackSizeEq(3, output_count),
            OpCode::Jump(0),
            OpCode::Brk,
        ]),
        input_connections: 1,
        output_connections: output_count,
    }
}
